"""
Connector model for orb.

This file is auto-generated from the connector definition at build time.
DO NOT EDIT MANUALLY - changes will be overwritten on next generation.
"""

from __future__ import annotations

from ._vendored.connector_sdk.types import (
    Action,
    AuthConfig,
    AuthType,
    ConnectorModel,
    EndpointDefinition,
    EntityDefinition,
)
from ._vendored.connector_sdk.schema.security import (
    AirbyteAuthConfig,
    AuthConfigFieldSpec,
)
from uuid import (
    UUID,
)

OrbConnectorModel: ConnectorModel = ConnectorModel(
    id=UUID('7f0455fb-4518-4ec0-b7a3-d808bf8081cc'),
    name='orb',
    version='0.1.2',
    base_url='https://api.billwithorb.com/v1',
    auth=AuthConfig(
        type=AuthType.BEARER,
        config={'header': 'Authorization', 'prefix': 'Bearer'},
        user_config_spec=AirbyteAuthConfig(
            title='API Key Authentication',
            type='object',
            required=['api_key'],
            properties={
                'api_key': AuthConfigFieldSpec(
                    title='API Key',
                    description='Your Orb API key',
                ),
            },
            auth_mapping={'token': '${api_key}'},
        ),
    ),
    entities=[
        EntityDefinition(
            name='customers',
            stream_name='customers',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/customers',
                    action=Action.LIST,
                    description='Returns a paginated list of customers',
                    query_params=['limit', 'cursor'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 50,
                        },
                        'cursor': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of customers',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Customer object',
                                    'properties': {
                                        'id': {'type': 'string', 'description': 'The unique identifier of the customer'},
                                        'external_customer_id': {
                                            'type': ['string', 'null'],
                                            'description': 'The ID of the customer in an external system',
                                        },
                                        'name': {
                                            'type': ['string', 'null'],
                                            'description': 'The name of the customer',
                                        },
                                        'email': {
                                            'type': ['string', 'null'],
                                            'description': 'The email address of the customer',
                                        },
                                        'created_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the customer was created',
                                        },
                                        'payment_provider': {
                                            'type': ['string', 'null'],
                                            'description': 'The payment provider used by the customer',
                                        },
                                        'payment_provider_id': {
                                            'type': ['string', 'null'],
                                            'description': "The ID of the customer in the payment provider's system",
                                        },
                                        'timezone': {
                                            'type': ['string', 'null'],
                                            'description': 'The timezone setting of the customer',
                                        },
                                        'shipping_address': {
                                            'oneOf': [
                                                {
                                                    'type': 'object',
                                                    'description': 'Address object',
                                                    'properties': {
                                                        'city': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The city of the address',
                                                        },
                                                        'country': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The country of the address',
                                                        },
                                                        'line1': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The first line of the address',
                                                        },
                                                        'line2': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The second line of the address',
                                                        },
                                                        'postal_code': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The postal code of the address',
                                                        },
                                                        'state': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The state or region of the address',
                                                        },
                                                    },
                                                },
                                                {'type': 'null'},
                                            ],
                                            'description': 'The shipping address of the customer',
                                        },
                                        'billing_address': {
                                            'oneOf': [
                                                {
                                                    'type': 'object',
                                                    'description': 'Address object',
                                                    'properties': {
                                                        'city': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The city of the address',
                                                        },
                                                        'country': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The country of the address',
                                                        },
                                                        'line1': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The first line of the address',
                                                        },
                                                        'line2': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The second line of the address',
                                                        },
                                                        'postal_code': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The postal code of the address',
                                                        },
                                                        'state': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The state or region of the address',
                                                        },
                                                    },
                                                },
                                                {'type': 'null'},
                                            ],
                                            'description': 'The billing address of the customer',
                                        },
                                        'balance': {
                                            'type': ['string', 'null'],
                                            'description': 'The current balance of the customer',
                                        },
                                        'currency': {
                                            'type': ['string', 'null'],
                                            'description': 'The currency of the customer',
                                        },
                                        'tax_id': {
                                            'type': ['object', 'null'],
                                            'description': 'Tax identification information',
                                            'properties': {
                                                'type': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The type of tax ID',
                                                },
                                                'value': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The value of the tax ID',
                                                },
                                                'country': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The country of the tax ID',
                                                },
                                            },
                                        },
                                        'auto_collection': {
                                            'type': ['boolean', 'null'],
                                            'description': 'Whether auto collection is enabled',
                                        },
                                        'metadata': {
                                            'type': ['object', 'null'],
                                            'description': 'Additional metadata for the customer',
                                            'additionalProperties': True,
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'customers',
                                    'x-airbyte-stream-name': 'customers',
                                },
                            },
                            'pagination_metadata': {
                                'type': 'object',
                                'description': 'Pagination metadata',
                                'properties': {
                                    'has_more': {
                                        'type': ['boolean', 'null'],
                                        'description': 'Whether there are more results',
                                    },
                                    'next_cursor': {
                                        'type': ['string', 'null'],
                                        'description': 'Cursor for the next page',
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_cursor': '$.pagination_metadata.next_cursor'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/customers/{customer_id}',
                    action=Action.GET,
                    description='Get a single customer by ID',
                    path_params=['customer_id'],
                    path_params_schema={
                        'customer_id': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Customer object',
                        'properties': {
                            'id': {'type': 'string', 'description': 'The unique identifier of the customer'},
                            'external_customer_id': {
                                'type': ['string', 'null'],
                                'description': 'The ID of the customer in an external system',
                            },
                            'name': {
                                'type': ['string', 'null'],
                                'description': 'The name of the customer',
                            },
                            'email': {
                                'type': ['string', 'null'],
                                'description': 'The email address of the customer',
                            },
                            'created_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the customer was created',
                            },
                            'payment_provider': {
                                'type': ['string', 'null'],
                                'description': 'The payment provider used by the customer',
                            },
                            'payment_provider_id': {
                                'type': ['string', 'null'],
                                'description': "The ID of the customer in the payment provider's system",
                            },
                            'timezone': {
                                'type': ['string', 'null'],
                                'description': 'The timezone setting of the customer',
                            },
                            'shipping_address': {
                                'oneOf': [
                                    {
                                        'type': 'object',
                                        'description': 'Address object',
                                        'properties': {
                                            'city': {
                                                'type': ['string', 'null'],
                                                'description': 'The city of the address',
                                            },
                                            'country': {
                                                'type': ['string', 'null'],
                                                'description': 'The country of the address',
                                            },
                                            'line1': {
                                                'type': ['string', 'null'],
                                                'description': 'The first line of the address',
                                            },
                                            'line2': {
                                                'type': ['string', 'null'],
                                                'description': 'The second line of the address',
                                            },
                                            'postal_code': {
                                                'type': ['string', 'null'],
                                                'description': 'The postal code of the address',
                                            },
                                            'state': {
                                                'type': ['string', 'null'],
                                                'description': 'The state or region of the address',
                                            },
                                        },
                                    },
                                    {'type': 'null'},
                                ],
                                'description': 'The shipping address of the customer',
                            },
                            'billing_address': {
                                'oneOf': [
                                    {
                                        'type': 'object',
                                        'description': 'Address object',
                                        'properties': {
                                            'city': {
                                                'type': ['string', 'null'],
                                                'description': 'The city of the address',
                                            },
                                            'country': {
                                                'type': ['string', 'null'],
                                                'description': 'The country of the address',
                                            },
                                            'line1': {
                                                'type': ['string', 'null'],
                                                'description': 'The first line of the address',
                                            },
                                            'line2': {
                                                'type': ['string', 'null'],
                                                'description': 'The second line of the address',
                                            },
                                            'postal_code': {
                                                'type': ['string', 'null'],
                                                'description': 'The postal code of the address',
                                            },
                                            'state': {
                                                'type': ['string', 'null'],
                                                'description': 'The state or region of the address',
                                            },
                                        },
                                    },
                                    {'type': 'null'},
                                ],
                                'description': 'The billing address of the customer',
                            },
                            'balance': {
                                'type': ['string', 'null'],
                                'description': 'The current balance of the customer',
                            },
                            'currency': {
                                'type': ['string', 'null'],
                                'description': 'The currency of the customer',
                            },
                            'tax_id': {
                                'type': ['object', 'null'],
                                'description': 'Tax identification information',
                                'properties': {
                                    'type': {
                                        'type': ['string', 'null'],
                                        'description': 'The type of tax ID',
                                    },
                                    'value': {
                                        'type': ['string', 'null'],
                                        'description': 'The value of the tax ID',
                                    },
                                    'country': {
                                        'type': ['string', 'null'],
                                        'description': 'The country of the tax ID',
                                    },
                                },
                            },
                            'auto_collection': {
                                'type': ['boolean', 'null'],
                                'description': 'Whether auto collection is enabled',
                            },
                            'metadata': {
                                'type': ['object', 'null'],
                                'description': 'Additional metadata for the customer',
                                'additionalProperties': True,
                            },
                        },
                        'required': ['id'],
                        'x-airbyte-entity-name': 'customers',
                        'x-airbyte-stream-name': 'customers',
                    },
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Customer object',
                'properties': {
                    'id': {'type': 'string', 'description': 'The unique identifier of the customer'},
                    'external_customer_id': {
                        'type': ['string', 'null'],
                        'description': 'The ID of the customer in an external system',
                    },
                    'name': {
                        'type': ['string', 'null'],
                        'description': 'The name of the customer',
                    },
                    'email': {
                        'type': ['string', 'null'],
                        'description': 'The email address of the customer',
                    },
                    'created_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the customer was created',
                    },
                    'payment_provider': {
                        'type': ['string', 'null'],
                        'description': 'The payment provider used by the customer',
                    },
                    'payment_provider_id': {
                        'type': ['string', 'null'],
                        'description': "The ID of the customer in the payment provider's system",
                    },
                    'timezone': {
                        'type': ['string', 'null'],
                        'description': 'The timezone setting of the customer',
                    },
                    'shipping_address': {
                        'oneOf': [
                            {'$ref': '#/components/schemas/Address'},
                            {'type': 'null'},
                        ],
                        'description': 'The shipping address of the customer',
                    },
                    'billing_address': {
                        'oneOf': [
                            {'$ref': '#/components/schemas/Address'},
                            {'type': 'null'},
                        ],
                        'description': 'The billing address of the customer',
                    },
                    'balance': {
                        'type': ['string', 'null'],
                        'description': 'The current balance of the customer',
                    },
                    'currency': {
                        'type': ['string', 'null'],
                        'description': 'The currency of the customer',
                    },
                    'tax_id': {
                        'type': ['object', 'null'],
                        'description': 'Tax identification information',
                        'properties': {
                            'type': {
                                'type': ['string', 'null'],
                                'description': 'The type of tax ID',
                            },
                            'value': {
                                'type': ['string', 'null'],
                                'description': 'The value of the tax ID',
                            },
                            'country': {
                                'type': ['string', 'null'],
                                'description': 'The country of the tax ID',
                            },
                        },
                    },
                    'auto_collection': {
                        'type': ['boolean', 'null'],
                        'description': 'Whether auto collection is enabled',
                    },
                    'metadata': {
                        'type': ['object', 'null'],
                        'description': 'Additional metadata for the customer',
                        'additionalProperties': True,
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'customers',
                'x-airbyte-stream-name': 'customers',
            },
        ),
        EntityDefinition(
            name='subscriptions',
            stream_name='subscriptions',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/subscriptions',
                    action=Action.LIST,
                    description='Returns a paginated list of subscriptions',
                    query_params=[
                        'limit',
                        'cursor',
                        'customer_id',
                        'external_customer_id',
                        'status',
                    ],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 50,
                        },
                        'cursor': {'type': 'string', 'required': False},
                        'customer_id': {'type': 'string', 'required': False},
                        'external_customer_id': {'type': 'string', 'required': False},
                        'status': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of subscriptions',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Subscription object',
                                    'properties': {
                                        'id': {'type': 'string', 'description': 'The unique identifier of the subscription'},
                                        'created_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the subscription was created',
                                        },
                                        'start_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the subscription starts',
                                        },
                                        'end_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the subscription ends',
                                        },
                                        'status': {
                                            'type': ['string', 'null'],
                                            'description': 'The current status of the subscription',
                                        },
                                        'customer': {
                                            'type': ['object', 'null'],
                                            'description': 'The customer associated with the subscription',
                                            'properties': {
                                                'id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The customer ID',
                                                },
                                                'external_customer_id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The external customer ID',
                                                },
                                            },
                                        },
                                        'plan': {
                                            'type': ['object', 'null'],
                                            'description': 'The plan associated with the subscription',
                                            'properties': {
                                                'id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The plan ID',
                                                },
                                                'name': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The plan name',
                                                },
                                            },
                                        },
                                        'current_billing_period_start_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The start date of the current billing period',
                                        },
                                        'current_billing_period_end_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The end date of the current billing period',
                                        },
                                        'active_plan_phase_order': {
                                            'type': ['integer', 'null'],
                                            'description': 'The order of the active plan phase',
                                        },
                                        'fixed_fee_quantity_schedule': {
                                            'type': ['array', 'null'],
                                            'description': 'The fixed fee quantity schedule',
                                            'items': {'type': 'object'},
                                        },
                                        'price_intervals': {
                                            'type': ['array', 'null'],
                                            'description': 'The price intervals for the subscription',
                                            'items': {'type': 'object'},
                                        },
                                        'redeemed_coupon': {
                                            'type': ['object', 'null'],
                                            'description': 'The redeemed coupon for the subscription',
                                        },
                                        'default_invoice_memo': {
                                            'type': ['string', 'null'],
                                            'description': 'The default invoice memo',
                                        },
                                        'auto_collection': {
                                            'type': ['boolean', 'null'],
                                            'description': 'Whether auto collection is enabled',
                                        },
                                        'net_terms': {
                                            'type': ['integer', 'null'],
                                            'description': 'The net terms for the subscription',
                                        },
                                        'invoicing_threshold': {
                                            'type': ['string', 'null'],
                                            'description': 'The invoicing threshold',
                                        },
                                        'metadata': {
                                            'type': ['object', 'null'],
                                            'description': 'Additional metadata for the subscription',
                                            'additionalProperties': True,
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'subscriptions',
                                    'x-airbyte-stream-name': 'subscriptions',
                                },
                            },
                            'pagination_metadata': {
                                'type': 'object',
                                'description': 'Pagination metadata',
                                'properties': {
                                    'has_more': {
                                        'type': ['boolean', 'null'],
                                        'description': 'Whether there are more results',
                                    },
                                    'next_cursor': {
                                        'type': ['string', 'null'],
                                        'description': 'Cursor for the next page',
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_cursor': '$.pagination_metadata.next_cursor'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/subscriptions/{subscription_id}',
                    action=Action.GET,
                    description='Get a single subscription by ID',
                    path_params=['subscription_id'],
                    path_params_schema={
                        'subscription_id': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Subscription object',
                        'properties': {
                            'id': {'type': 'string', 'description': 'The unique identifier of the subscription'},
                            'created_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the subscription was created',
                            },
                            'start_date': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the subscription starts',
                            },
                            'end_date': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the subscription ends',
                            },
                            'status': {
                                'type': ['string', 'null'],
                                'description': 'The current status of the subscription',
                            },
                            'customer': {
                                'type': ['object', 'null'],
                                'description': 'The customer associated with the subscription',
                                'properties': {
                                    'id': {
                                        'type': ['string', 'null'],
                                        'description': 'The customer ID',
                                    },
                                    'external_customer_id': {
                                        'type': ['string', 'null'],
                                        'description': 'The external customer ID',
                                    },
                                },
                            },
                            'plan': {
                                'type': ['object', 'null'],
                                'description': 'The plan associated with the subscription',
                                'properties': {
                                    'id': {
                                        'type': ['string', 'null'],
                                        'description': 'The plan ID',
                                    },
                                    'name': {
                                        'type': ['string', 'null'],
                                        'description': 'The plan name',
                                    },
                                },
                            },
                            'current_billing_period_start_date': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The start date of the current billing period',
                            },
                            'current_billing_period_end_date': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The end date of the current billing period',
                            },
                            'active_plan_phase_order': {
                                'type': ['integer', 'null'],
                                'description': 'The order of the active plan phase',
                            },
                            'fixed_fee_quantity_schedule': {
                                'type': ['array', 'null'],
                                'description': 'The fixed fee quantity schedule',
                                'items': {'type': 'object'},
                            },
                            'price_intervals': {
                                'type': ['array', 'null'],
                                'description': 'The price intervals for the subscription',
                                'items': {'type': 'object'},
                            },
                            'redeemed_coupon': {
                                'type': ['object', 'null'],
                                'description': 'The redeemed coupon for the subscription',
                            },
                            'default_invoice_memo': {
                                'type': ['string', 'null'],
                                'description': 'The default invoice memo',
                            },
                            'auto_collection': {
                                'type': ['boolean', 'null'],
                                'description': 'Whether auto collection is enabled',
                            },
                            'net_terms': {
                                'type': ['integer', 'null'],
                                'description': 'The net terms for the subscription',
                            },
                            'invoicing_threshold': {
                                'type': ['string', 'null'],
                                'description': 'The invoicing threshold',
                            },
                            'metadata': {
                                'type': ['object', 'null'],
                                'description': 'Additional metadata for the subscription',
                                'additionalProperties': True,
                            },
                        },
                        'required': ['id'],
                        'x-airbyte-entity-name': 'subscriptions',
                        'x-airbyte-stream-name': 'subscriptions',
                    },
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Subscription object',
                'properties': {
                    'id': {'type': 'string', 'description': 'The unique identifier of the subscription'},
                    'created_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the subscription was created',
                    },
                    'start_date': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the subscription starts',
                    },
                    'end_date': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the subscription ends',
                    },
                    'status': {
                        'type': ['string', 'null'],
                        'description': 'The current status of the subscription',
                    },
                    'customer': {
                        'type': ['object', 'null'],
                        'description': 'The customer associated with the subscription',
                        'properties': {
                            'id': {
                                'type': ['string', 'null'],
                                'description': 'The customer ID',
                            },
                            'external_customer_id': {
                                'type': ['string', 'null'],
                                'description': 'The external customer ID',
                            },
                        },
                    },
                    'plan': {
                        'type': ['object', 'null'],
                        'description': 'The plan associated with the subscription',
                        'properties': {
                            'id': {
                                'type': ['string', 'null'],
                                'description': 'The plan ID',
                            },
                            'name': {
                                'type': ['string', 'null'],
                                'description': 'The plan name',
                            },
                        },
                    },
                    'current_billing_period_start_date': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The start date of the current billing period',
                    },
                    'current_billing_period_end_date': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The end date of the current billing period',
                    },
                    'active_plan_phase_order': {
                        'type': ['integer', 'null'],
                        'description': 'The order of the active plan phase',
                    },
                    'fixed_fee_quantity_schedule': {
                        'type': ['array', 'null'],
                        'description': 'The fixed fee quantity schedule',
                        'items': {'type': 'object'},
                    },
                    'price_intervals': {
                        'type': ['array', 'null'],
                        'description': 'The price intervals for the subscription',
                        'items': {'type': 'object'},
                    },
                    'redeemed_coupon': {
                        'type': ['object', 'null'],
                        'description': 'The redeemed coupon for the subscription',
                    },
                    'default_invoice_memo': {
                        'type': ['string', 'null'],
                        'description': 'The default invoice memo',
                    },
                    'auto_collection': {
                        'type': ['boolean', 'null'],
                        'description': 'Whether auto collection is enabled',
                    },
                    'net_terms': {
                        'type': ['integer', 'null'],
                        'description': 'The net terms for the subscription',
                    },
                    'invoicing_threshold': {
                        'type': ['string', 'null'],
                        'description': 'The invoicing threshold',
                    },
                    'metadata': {
                        'type': ['object', 'null'],
                        'description': 'Additional metadata for the subscription',
                        'additionalProperties': True,
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'subscriptions',
                'x-airbyte-stream-name': 'subscriptions',
            },
        ),
        EntityDefinition(
            name='plans',
            stream_name='plans',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/plans',
                    action=Action.LIST,
                    description='Returns a paginated list of plans',
                    query_params=['limit', 'cursor'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 50,
                        },
                        'cursor': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of plans',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Plan object',
                                    'properties': {
                                        'id': {'type': 'string', 'description': 'The unique identifier of the plan'},
                                        'created_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the plan was created',
                                        },
                                        'name': {
                                            'type': ['string', 'null'],
                                            'description': 'The name of the plan',
                                        },
                                        'description': {
                                            'type': ['string', 'null'],
                                            'description': 'A description of the plan',
                                        },
                                        'status': {
                                            'type': ['string', 'null'],
                                            'description': 'The status of the plan',
                                        },
                                        'default_invoice_memo': {
                                            'type': ['string', 'null'],
                                            'description': 'The default invoice memo for the plan',
                                        },
                                        'net_terms': {
                                            'type': ['integer', 'null'],
                                            'description': 'The net terms for the plan',
                                        },
                                        'currency': {
                                            'type': ['string', 'null'],
                                            'description': 'The currency of the plan',
                                        },
                                        'prices': {
                                            'type': ['array', 'null'],
                                            'description': 'The pricing options for the plan',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'id': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The unique identifier of the price',
                                                    },
                                                    'name': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The name of the price',
                                                    },
                                                    'price_type': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The type of price',
                                                    },
                                                    'model_type': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The model type of the price',
                                                    },
                                                    'currency': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The currency of the price',
                                                    },
                                                },
                                            },
                                        },
                                        'product': {
                                            'type': ['object', 'null'],
                                            'description': 'The product associated with the plan',
                                            'properties': {
                                                'id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The product ID',
                                                },
                                                'name': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The product name',
                                                },
                                            },
                                        },
                                        'minimum': {
                                            'type': ['object', 'null'],
                                            'description': 'The minimum configuration for the plan',
                                        },
                                        'maximum': {
                                            'type': ['object', 'null'],
                                            'description': 'The maximum configuration for the plan',
                                        },
                                        'discount': {
                                            'type': ['object', 'null'],
                                            'description': 'The discount configuration for the plan',
                                        },
                                        'trial_config': {
                                            'type': ['object', 'null'],
                                            'description': 'The trial configuration for the plan',
                                        },
                                        'plan_phases': {
                                            'type': ['array', 'null'],
                                            'description': 'The phases of the plan',
                                            'items': {'type': 'object'},
                                        },
                                        'external_plan_id': {
                                            'type': ['string', 'null'],
                                            'description': 'The external plan ID',
                                        },
                                        'invoicing_currency': {
                                            'type': ['string', 'null'],
                                            'description': 'The invoicing currency',
                                        },
                                        'metadata': {
                                            'type': ['object', 'null'],
                                            'description': 'Additional metadata for the plan',
                                            'additionalProperties': True,
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'plans',
                                    'x-airbyte-stream-name': 'plans',
                                },
                            },
                            'pagination_metadata': {
                                'type': 'object',
                                'description': 'Pagination metadata',
                                'properties': {
                                    'has_more': {
                                        'type': ['boolean', 'null'],
                                        'description': 'Whether there are more results',
                                    },
                                    'next_cursor': {
                                        'type': ['string', 'null'],
                                        'description': 'Cursor for the next page',
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_cursor': '$.pagination_metadata.next_cursor'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/plans/{plan_id}',
                    action=Action.GET,
                    description='Get a single plan by ID',
                    path_params=['plan_id'],
                    path_params_schema={
                        'plan_id': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Plan object',
                        'properties': {
                            'id': {'type': 'string', 'description': 'The unique identifier of the plan'},
                            'created_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the plan was created',
                            },
                            'name': {
                                'type': ['string', 'null'],
                                'description': 'The name of the plan',
                            },
                            'description': {
                                'type': ['string', 'null'],
                                'description': 'A description of the plan',
                            },
                            'status': {
                                'type': ['string', 'null'],
                                'description': 'The status of the plan',
                            },
                            'default_invoice_memo': {
                                'type': ['string', 'null'],
                                'description': 'The default invoice memo for the plan',
                            },
                            'net_terms': {
                                'type': ['integer', 'null'],
                                'description': 'The net terms for the plan',
                            },
                            'currency': {
                                'type': ['string', 'null'],
                                'description': 'The currency of the plan',
                            },
                            'prices': {
                                'type': ['array', 'null'],
                                'description': 'The pricing options for the plan',
                                'items': {
                                    'type': 'object',
                                    'properties': {
                                        'id': {
                                            'type': ['string', 'null'],
                                            'description': 'The unique identifier of the price',
                                        },
                                        'name': {
                                            'type': ['string', 'null'],
                                            'description': 'The name of the price',
                                        },
                                        'price_type': {
                                            'type': ['string', 'null'],
                                            'description': 'The type of price',
                                        },
                                        'model_type': {
                                            'type': ['string', 'null'],
                                            'description': 'The model type of the price',
                                        },
                                        'currency': {
                                            'type': ['string', 'null'],
                                            'description': 'The currency of the price',
                                        },
                                    },
                                },
                            },
                            'product': {
                                'type': ['object', 'null'],
                                'description': 'The product associated with the plan',
                                'properties': {
                                    'id': {
                                        'type': ['string', 'null'],
                                        'description': 'The product ID',
                                    },
                                    'name': {
                                        'type': ['string', 'null'],
                                        'description': 'The product name',
                                    },
                                },
                            },
                            'minimum': {
                                'type': ['object', 'null'],
                                'description': 'The minimum configuration for the plan',
                            },
                            'maximum': {
                                'type': ['object', 'null'],
                                'description': 'The maximum configuration for the plan',
                            },
                            'discount': {
                                'type': ['object', 'null'],
                                'description': 'The discount configuration for the plan',
                            },
                            'trial_config': {
                                'type': ['object', 'null'],
                                'description': 'The trial configuration for the plan',
                            },
                            'plan_phases': {
                                'type': ['array', 'null'],
                                'description': 'The phases of the plan',
                                'items': {'type': 'object'},
                            },
                            'external_plan_id': {
                                'type': ['string', 'null'],
                                'description': 'The external plan ID',
                            },
                            'invoicing_currency': {
                                'type': ['string', 'null'],
                                'description': 'The invoicing currency',
                            },
                            'metadata': {
                                'type': ['object', 'null'],
                                'description': 'Additional metadata for the plan',
                                'additionalProperties': True,
                            },
                        },
                        'required': ['id'],
                        'x-airbyte-entity-name': 'plans',
                        'x-airbyte-stream-name': 'plans',
                    },
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Plan object',
                'properties': {
                    'id': {'type': 'string', 'description': 'The unique identifier of the plan'},
                    'created_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the plan was created',
                    },
                    'name': {
                        'type': ['string', 'null'],
                        'description': 'The name of the plan',
                    },
                    'description': {
                        'type': ['string', 'null'],
                        'description': 'A description of the plan',
                    },
                    'status': {
                        'type': ['string', 'null'],
                        'description': 'The status of the plan',
                    },
                    'default_invoice_memo': {
                        'type': ['string', 'null'],
                        'description': 'The default invoice memo for the plan',
                    },
                    'net_terms': {
                        'type': ['integer', 'null'],
                        'description': 'The net terms for the plan',
                    },
                    'currency': {
                        'type': ['string', 'null'],
                        'description': 'The currency of the plan',
                    },
                    'prices': {
                        'type': ['array', 'null'],
                        'description': 'The pricing options for the plan',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'id': {
                                    'type': ['string', 'null'],
                                    'description': 'The unique identifier of the price',
                                },
                                'name': {
                                    'type': ['string', 'null'],
                                    'description': 'The name of the price',
                                },
                                'price_type': {
                                    'type': ['string', 'null'],
                                    'description': 'The type of price',
                                },
                                'model_type': {
                                    'type': ['string', 'null'],
                                    'description': 'The model type of the price',
                                },
                                'currency': {
                                    'type': ['string', 'null'],
                                    'description': 'The currency of the price',
                                },
                            },
                        },
                    },
                    'product': {
                        'type': ['object', 'null'],
                        'description': 'The product associated with the plan',
                        'properties': {
                            'id': {
                                'type': ['string', 'null'],
                                'description': 'The product ID',
                            },
                            'name': {
                                'type': ['string', 'null'],
                                'description': 'The product name',
                            },
                        },
                    },
                    'minimum': {
                        'type': ['object', 'null'],
                        'description': 'The minimum configuration for the plan',
                    },
                    'maximum': {
                        'type': ['object', 'null'],
                        'description': 'The maximum configuration for the plan',
                    },
                    'discount': {
                        'type': ['object', 'null'],
                        'description': 'The discount configuration for the plan',
                    },
                    'trial_config': {
                        'type': ['object', 'null'],
                        'description': 'The trial configuration for the plan',
                    },
                    'plan_phases': {
                        'type': ['array', 'null'],
                        'description': 'The phases of the plan',
                        'items': {'type': 'object'},
                    },
                    'external_plan_id': {
                        'type': ['string', 'null'],
                        'description': 'The external plan ID',
                    },
                    'invoicing_currency': {
                        'type': ['string', 'null'],
                        'description': 'The invoicing currency',
                    },
                    'metadata': {
                        'type': ['object', 'null'],
                        'description': 'Additional metadata for the plan',
                        'additionalProperties': True,
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'plans',
                'x-airbyte-stream-name': 'plans',
            },
        ),
        EntityDefinition(
            name='invoices',
            stream_name='invoices',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/invoices',
                    action=Action.LIST,
                    description='Returns a paginated list of invoices',
                    query_params=[
                        'limit',
                        'cursor',
                        'customer_id',
                        'external_customer_id',
                        'subscription_id',
                        'invoice_date_gt',
                        'invoice_date_gte',
                        'invoice_date_lt',
                        'invoice_date_lte',
                        'status',
                    ],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 50,
                        },
                        'cursor': {'type': 'string', 'required': False},
                        'customer_id': {'type': 'string', 'required': False},
                        'external_customer_id': {'type': 'string', 'required': False},
                        'subscription_id': {'type': 'string', 'required': False},
                        'invoice_date_gt': {'type': 'string', 'required': False},
                        'invoice_date_gte': {'type': 'string', 'required': False},
                        'invoice_date_lt': {'type': 'string', 'required': False},
                        'invoice_date_lte': {'type': 'string', 'required': False},
                        'status': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of invoices',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Invoice object',
                                    'properties': {
                                        'id': {'type': 'string', 'description': 'The unique identifier of the invoice'},
                                        'created_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the invoice was created',
                                        },
                                        'invoice_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date of the invoice',
                                        },
                                        'due_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The due date for the invoice',
                                        },
                                        'invoice_pdf': {
                                            'type': ['string', 'null'],
                                            'description': 'The URL to download the PDF version of the invoice',
                                        },
                                        'subtotal': {
                                            'type': ['string', 'null'],
                                            'description': 'The subtotal amount of the invoice',
                                        },
                                        'total': {
                                            'type': ['string', 'null'],
                                            'description': 'The total amount of the invoice',
                                        },
                                        'amount_due': {
                                            'type': ['string', 'null'],
                                            'description': 'The amount due on the invoice',
                                        },
                                        'status': {
                                            'type': ['string', 'null'],
                                            'description': 'The current status of the invoice',
                                        },
                                        'memo': {
                                            'type': ['string', 'null'],
                                            'description': 'Any additional notes or comments on the invoice',
                                        },
                                        'issue_failed_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when issuing the invoice failed',
                                        },
                                        'sync_failed_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when syncing the invoice failed',
                                        },
                                        'payment_failed_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when payment failed',
                                        },
                                        'payment_started_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when payment started',
                                        },
                                        'voided_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the invoice was voided',
                                        },
                                        'paid_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the invoice was paid',
                                        },
                                        'issued_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date and time when the invoice was issued',
                                        },
                                        'hosted_invoice_url': {
                                            'type': ['string', 'null'],
                                            'description': 'The URL to view the hosted invoice',
                                        },
                                        'line_items': {
                                            'type': ['array', 'null'],
                                            'description': 'The line items on the invoice',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'id': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The unique identifier of the line item',
                                                    },
                                                    'quantity': {
                                                        'type': ['number', 'null'],
                                                        'description': 'The quantity of the line item',
                                                    },
                                                    'amount': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The amount of the line item',
                                                    },
                                                    'name': {
                                                        'type': ['string', 'null'],
                                                        'description': 'The name of the line item',
                                                    },
                                                    'start_date': {
                                                        'type': ['string', 'null'],
                                                        'format': 'date-time',
                                                        'description': 'The start date of the line item',
                                                    },
                                                    'end_date': {
                                                        'type': ['string', 'null'],
                                                        'format': 'date-time',
                                                        'description': 'The end date of the line item',
                                                    },
                                                },
                                            },
                                        },
                                        'subscription': {
                                            'type': ['object', 'null'],
                                            'description': 'The subscription associated with the invoice',
                                            'properties': {
                                                'id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The subscription ID',
                                                },
                                            },
                                        },
                                        'customer': {
                                            'type': ['object', 'null'],
                                            'description': 'The customer associated with the invoice',
                                            'properties': {
                                                'id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The customer ID',
                                                },
                                                'external_customer_id': {
                                                    'type': ['string', 'null'],
                                                    'description': 'The external customer ID',
                                                },
                                            },
                                        },
                                        'currency': {
                                            'type': ['string', 'null'],
                                            'description': 'The currency of the invoice',
                                        },
                                        'discount': {
                                            'type': ['object', 'null'],
                                            'description': 'The discount applied to the invoice',
                                        },
                                        'minimum': {
                                            'type': ['object', 'null'],
                                            'description': 'The minimum configuration for the invoice',
                                        },
                                        'maximum': {
                                            'type': ['object', 'null'],
                                            'description': 'The maximum configuration for the invoice',
                                        },
                                        'credit_notes': {
                                            'type': ['array', 'null'],
                                            'description': 'Credit notes associated with the invoice',
                                            'items': {'type': 'object'},
                                        },
                                        'will_auto_issue': {
                                            'type': ['boolean', 'null'],
                                            'description': 'Whether the invoice will auto issue',
                                        },
                                        'eligible_to_issue_at': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The date when the invoice is eligible to issue',
                                        },
                                        'customer_balance_transactions': {
                                            'type': ['array', 'null'],
                                            'description': 'Customer balance transactions',
                                            'items': {'type': 'object'},
                                        },
                                        'auto_collection': {
                                            'type': ['object', 'null'],
                                            'description': 'Auto collection settings',
                                        },
                                        'invoice_number': {
                                            'type': ['string', 'null'],
                                            'description': 'The invoice number',
                                        },
                                        'billing_address': {
                                            'oneOf': [
                                                {
                                                    'type': 'object',
                                                    'description': 'Address object',
                                                    'properties': {
                                                        'city': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The city of the address',
                                                        },
                                                        'country': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The country of the address',
                                                        },
                                                        'line1': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The first line of the address',
                                                        },
                                                        'line2': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The second line of the address',
                                                        },
                                                        'postal_code': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The postal code of the address',
                                                        },
                                                        'state': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The state or region of the address',
                                                        },
                                                    },
                                                },
                                                {'type': 'null'},
                                            ],
                                            'description': 'The billing address on the invoice',
                                        },
                                        'shipping_address': {
                                            'oneOf': [
                                                {
                                                    'type': 'object',
                                                    'description': 'Address object',
                                                    'properties': {
                                                        'city': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The city of the address',
                                                        },
                                                        'country': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The country of the address',
                                                        },
                                                        'line1': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The first line of the address',
                                                        },
                                                        'line2': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The second line of the address',
                                                        },
                                                        'postal_code': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The postal code of the address',
                                                        },
                                                        'state': {
                                                            'type': ['string', 'null'],
                                                            'description': 'The state or region of the address',
                                                        },
                                                    },
                                                },
                                                {'type': 'null'},
                                            ],
                                            'description': 'The shipping address on the invoice',
                                        },
                                        'metadata': {
                                            'type': ['object', 'null'],
                                            'description': 'Additional metadata for the invoice',
                                            'additionalProperties': True,
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'invoices',
                                    'x-airbyte-stream-name': 'invoices',
                                },
                            },
                            'pagination_metadata': {
                                'type': 'object',
                                'description': 'Pagination metadata',
                                'properties': {
                                    'has_more': {
                                        'type': ['boolean', 'null'],
                                        'description': 'Whether there are more results',
                                    },
                                    'next_cursor': {
                                        'type': ['string', 'null'],
                                        'description': 'Cursor for the next page',
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_cursor': '$.pagination_metadata.next_cursor'},
                    untested=True,
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/invoices/{invoice_id}',
                    action=Action.GET,
                    description='Get a single invoice by ID',
                    path_params=['invoice_id'],
                    path_params_schema={
                        'invoice_id': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Invoice object',
                        'properties': {
                            'id': {'type': 'string', 'description': 'The unique identifier of the invoice'},
                            'created_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the invoice was created',
                            },
                            'invoice_date': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date of the invoice',
                            },
                            'due_date': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The due date for the invoice',
                            },
                            'invoice_pdf': {
                                'type': ['string', 'null'],
                                'description': 'The URL to download the PDF version of the invoice',
                            },
                            'subtotal': {
                                'type': ['string', 'null'],
                                'description': 'The subtotal amount of the invoice',
                            },
                            'total': {
                                'type': ['string', 'null'],
                                'description': 'The total amount of the invoice',
                            },
                            'amount_due': {
                                'type': ['string', 'null'],
                                'description': 'The amount due on the invoice',
                            },
                            'status': {
                                'type': ['string', 'null'],
                                'description': 'The current status of the invoice',
                            },
                            'memo': {
                                'type': ['string', 'null'],
                                'description': 'Any additional notes or comments on the invoice',
                            },
                            'issue_failed_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when issuing the invoice failed',
                            },
                            'sync_failed_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when syncing the invoice failed',
                            },
                            'payment_failed_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when payment failed',
                            },
                            'payment_started_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when payment started',
                            },
                            'voided_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the invoice was voided',
                            },
                            'paid_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the invoice was paid',
                            },
                            'issued_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date and time when the invoice was issued',
                            },
                            'hosted_invoice_url': {
                                'type': ['string', 'null'],
                                'description': 'The URL to view the hosted invoice',
                            },
                            'line_items': {
                                'type': ['array', 'null'],
                                'description': 'The line items on the invoice',
                                'items': {
                                    'type': 'object',
                                    'properties': {
                                        'id': {
                                            'type': ['string', 'null'],
                                            'description': 'The unique identifier of the line item',
                                        },
                                        'quantity': {
                                            'type': ['number', 'null'],
                                            'description': 'The quantity of the line item',
                                        },
                                        'amount': {
                                            'type': ['string', 'null'],
                                            'description': 'The amount of the line item',
                                        },
                                        'name': {
                                            'type': ['string', 'null'],
                                            'description': 'The name of the line item',
                                        },
                                        'start_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The start date of the line item',
                                        },
                                        'end_date': {
                                            'type': ['string', 'null'],
                                            'format': 'date-time',
                                            'description': 'The end date of the line item',
                                        },
                                    },
                                },
                            },
                            'subscription': {
                                'type': ['object', 'null'],
                                'description': 'The subscription associated with the invoice',
                                'properties': {
                                    'id': {
                                        'type': ['string', 'null'],
                                        'description': 'The subscription ID',
                                    },
                                },
                            },
                            'customer': {
                                'type': ['object', 'null'],
                                'description': 'The customer associated with the invoice',
                                'properties': {
                                    'id': {
                                        'type': ['string', 'null'],
                                        'description': 'The customer ID',
                                    },
                                    'external_customer_id': {
                                        'type': ['string', 'null'],
                                        'description': 'The external customer ID',
                                    },
                                },
                            },
                            'currency': {
                                'type': ['string', 'null'],
                                'description': 'The currency of the invoice',
                            },
                            'discount': {
                                'type': ['object', 'null'],
                                'description': 'The discount applied to the invoice',
                            },
                            'minimum': {
                                'type': ['object', 'null'],
                                'description': 'The minimum configuration for the invoice',
                            },
                            'maximum': {
                                'type': ['object', 'null'],
                                'description': 'The maximum configuration for the invoice',
                            },
                            'credit_notes': {
                                'type': ['array', 'null'],
                                'description': 'Credit notes associated with the invoice',
                                'items': {'type': 'object'},
                            },
                            'will_auto_issue': {
                                'type': ['boolean', 'null'],
                                'description': 'Whether the invoice will auto issue',
                            },
                            'eligible_to_issue_at': {
                                'type': ['string', 'null'],
                                'format': 'date-time',
                                'description': 'The date when the invoice is eligible to issue',
                            },
                            'customer_balance_transactions': {
                                'type': ['array', 'null'],
                                'description': 'Customer balance transactions',
                                'items': {'type': 'object'},
                            },
                            'auto_collection': {
                                'type': ['object', 'null'],
                                'description': 'Auto collection settings',
                            },
                            'invoice_number': {
                                'type': ['string', 'null'],
                                'description': 'The invoice number',
                            },
                            'billing_address': {
                                'oneOf': [
                                    {
                                        'type': 'object',
                                        'description': 'Address object',
                                        'properties': {
                                            'city': {
                                                'type': ['string', 'null'],
                                                'description': 'The city of the address',
                                            },
                                            'country': {
                                                'type': ['string', 'null'],
                                                'description': 'The country of the address',
                                            },
                                            'line1': {
                                                'type': ['string', 'null'],
                                                'description': 'The first line of the address',
                                            },
                                            'line2': {
                                                'type': ['string', 'null'],
                                                'description': 'The second line of the address',
                                            },
                                            'postal_code': {
                                                'type': ['string', 'null'],
                                                'description': 'The postal code of the address',
                                            },
                                            'state': {
                                                'type': ['string', 'null'],
                                                'description': 'The state or region of the address',
                                            },
                                        },
                                    },
                                    {'type': 'null'},
                                ],
                                'description': 'The billing address on the invoice',
                            },
                            'shipping_address': {
                                'oneOf': [
                                    {
                                        'type': 'object',
                                        'description': 'Address object',
                                        'properties': {
                                            'city': {
                                                'type': ['string', 'null'],
                                                'description': 'The city of the address',
                                            },
                                            'country': {
                                                'type': ['string', 'null'],
                                                'description': 'The country of the address',
                                            },
                                            'line1': {
                                                'type': ['string', 'null'],
                                                'description': 'The first line of the address',
                                            },
                                            'line2': {
                                                'type': ['string', 'null'],
                                                'description': 'The second line of the address',
                                            },
                                            'postal_code': {
                                                'type': ['string', 'null'],
                                                'description': 'The postal code of the address',
                                            },
                                            'state': {
                                                'type': ['string', 'null'],
                                                'description': 'The state or region of the address',
                                            },
                                        },
                                    },
                                    {'type': 'null'},
                                ],
                                'description': 'The shipping address on the invoice',
                            },
                            'metadata': {
                                'type': ['object', 'null'],
                                'description': 'Additional metadata for the invoice',
                                'additionalProperties': True,
                            },
                        },
                        'required': ['id'],
                        'x-airbyte-entity-name': 'invoices',
                        'x-airbyte-stream-name': 'invoices',
                    },
                    untested=True,
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Invoice object',
                'properties': {
                    'id': {'type': 'string', 'description': 'The unique identifier of the invoice'},
                    'created_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the invoice was created',
                    },
                    'invoice_date': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date of the invoice',
                    },
                    'due_date': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The due date for the invoice',
                    },
                    'invoice_pdf': {
                        'type': ['string', 'null'],
                        'description': 'The URL to download the PDF version of the invoice',
                    },
                    'subtotal': {
                        'type': ['string', 'null'],
                        'description': 'The subtotal amount of the invoice',
                    },
                    'total': {
                        'type': ['string', 'null'],
                        'description': 'The total amount of the invoice',
                    },
                    'amount_due': {
                        'type': ['string', 'null'],
                        'description': 'The amount due on the invoice',
                    },
                    'status': {
                        'type': ['string', 'null'],
                        'description': 'The current status of the invoice',
                    },
                    'memo': {
                        'type': ['string', 'null'],
                        'description': 'Any additional notes or comments on the invoice',
                    },
                    'issue_failed_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when issuing the invoice failed',
                    },
                    'sync_failed_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when syncing the invoice failed',
                    },
                    'payment_failed_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when payment failed',
                    },
                    'payment_started_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when payment started',
                    },
                    'voided_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the invoice was voided',
                    },
                    'paid_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the invoice was paid',
                    },
                    'issued_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date and time when the invoice was issued',
                    },
                    'hosted_invoice_url': {
                        'type': ['string', 'null'],
                        'description': 'The URL to view the hosted invoice',
                    },
                    'line_items': {
                        'type': ['array', 'null'],
                        'description': 'The line items on the invoice',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'id': {
                                    'type': ['string', 'null'],
                                    'description': 'The unique identifier of the line item',
                                },
                                'quantity': {
                                    'type': ['number', 'null'],
                                    'description': 'The quantity of the line item',
                                },
                                'amount': {
                                    'type': ['string', 'null'],
                                    'description': 'The amount of the line item',
                                },
                                'name': {
                                    'type': ['string', 'null'],
                                    'description': 'The name of the line item',
                                },
                                'start_date': {
                                    'type': ['string', 'null'],
                                    'format': 'date-time',
                                    'description': 'The start date of the line item',
                                },
                                'end_date': {
                                    'type': ['string', 'null'],
                                    'format': 'date-time',
                                    'description': 'The end date of the line item',
                                },
                            },
                        },
                    },
                    'subscription': {
                        'type': ['object', 'null'],
                        'description': 'The subscription associated with the invoice',
                        'properties': {
                            'id': {
                                'type': ['string', 'null'],
                                'description': 'The subscription ID',
                            },
                        },
                    },
                    'customer': {
                        'type': ['object', 'null'],
                        'description': 'The customer associated with the invoice',
                        'properties': {
                            'id': {
                                'type': ['string', 'null'],
                                'description': 'The customer ID',
                            },
                            'external_customer_id': {
                                'type': ['string', 'null'],
                                'description': 'The external customer ID',
                            },
                        },
                    },
                    'currency': {
                        'type': ['string', 'null'],
                        'description': 'The currency of the invoice',
                    },
                    'discount': {
                        'type': ['object', 'null'],
                        'description': 'The discount applied to the invoice',
                    },
                    'minimum': {
                        'type': ['object', 'null'],
                        'description': 'The minimum configuration for the invoice',
                    },
                    'maximum': {
                        'type': ['object', 'null'],
                        'description': 'The maximum configuration for the invoice',
                    },
                    'credit_notes': {
                        'type': ['array', 'null'],
                        'description': 'Credit notes associated with the invoice',
                        'items': {'type': 'object'},
                    },
                    'will_auto_issue': {
                        'type': ['boolean', 'null'],
                        'description': 'Whether the invoice will auto issue',
                    },
                    'eligible_to_issue_at': {
                        'type': ['string', 'null'],
                        'format': 'date-time',
                        'description': 'The date when the invoice is eligible to issue',
                    },
                    'customer_balance_transactions': {
                        'type': ['array', 'null'],
                        'description': 'Customer balance transactions',
                        'items': {'type': 'object'},
                    },
                    'auto_collection': {
                        'type': ['object', 'null'],
                        'description': 'Auto collection settings',
                    },
                    'invoice_number': {
                        'type': ['string', 'null'],
                        'description': 'The invoice number',
                    },
                    'billing_address': {
                        'oneOf': [
                            {'$ref': '#/components/schemas/Address'},
                            {'type': 'null'},
                        ],
                        'description': 'The billing address on the invoice',
                    },
                    'shipping_address': {
                        'oneOf': [
                            {'$ref': '#/components/schemas/Address'},
                            {'type': 'null'},
                        ],
                        'description': 'The shipping address on the invoice',
                    },
                    'metadata': {
                        'type': ['object', 'null'],
                        'description': 'Additional metadata for the invoice',
                        'additionalProperties': True,
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'invoices',
                'x-airbyte-stream-name': 'invoices',
            },
        ),
    ],
    search_field_paths={
        'customers': [
            'id',
            'external_customer_id',
            'name',
            'email',
            'created_at',
            'payment_provider',
            'payment_provider_id',
            'timezone',
            'shipping_address',
            'billing_address',
            'balance',
            'currency',
            'auto_collection',
            'metadata',
        ],
        'subscriptions': [
            'id',
            'created_at',
            'start_date',
            'end_date',
            'status',
            'customer',
            'plan',
            'current_billing_period_start_date',
            'current_billing_period_end_date',
            'auto_collection',
            'net_terms',
            'metadata',
        ],
        'plans': [
            'id',
            'created_at',
            'name',
            'description',
            'status',
            'currency',
            'prices',
            'prices[]',
            'product',
            'external_plan_id',
            'metadata',
        ],
        'invoices': [
            'id',
            'created_at',
            'invoice_date',
            'due_date',
            'invoice_pdf',
            'subtotal',
            'total',
            'amount_due',
            'status',
            'memo',
            'paid_at',
            'issued_at',
            'hosted_invoice_url',
            'line_items',
            'line_items[]',
            'subscription',
            'customer',
            'currency',
            'invoice_number',
            'metadata',
        ],
    },
)


"""HTTP-related exceptions for the Airbyte SDK."""

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .response import HTTPResponse


class HTTPClientError(Exception):
    """Base exception for HTTP client errors."""

    pass


class HTTPStatusError(HTTPClientError):
    """Raised when an HTTP response has a 4xx or 5xx status code.

    This is the base exception for status code errors and is raised by
    HTTPResponse.raise_for_status().
    """

    def __init__(
        self,
        status_code: int,
        message: str,
        response: "HTTPResponse | None" = None,
    ) -> None:
        """Initialize HTTP status error.

        Args:
            status_code: The HTTP status code (e.g., 404, 500)
            message: Error message describing the issue
            response: Optional HTTPResponse object for accessing details
        """
        super().__init__(message)
        self.status_code = status_code
        self.response = response


class AuthenticationError(HTTPStatusError):
    """Raised when authentication credentials are missing or invalid (401, 403)."""

    def __init__(
        self,
        message: str,
        status_code: int = 401,
        response: "HTTPResponse | None" = None,
    ) -> None:
        """Initialize authentication error.

        Args:
            message: Error message describing the authentication issue
            status_code: HTTP status code (401 or 403)
            response: Optional HTTPResponse object for accessing details
        """
        super().__init__(status_code=status_code, message=message, response=response)


class RateLimitError(HTTPStatusError):
    """Raised when API rate limit is exceeded (429 response)."""

    def __init__(
        self,
        message: str,
        retry_after: int | None = None,
        response: "HTTPResponse | None" = None,
    ) -> None:
        """Initialize rate limit error.

        Args:
            message: Error message describing the rate limit
            retry_after: Seconds to wait before retrying (from Retry-After header)
            response: Optional HTTPResponse object for accessing details
        """
        super().__init__(status_code=429, message=message, response=response)
        self.retry_after = retry_after


class NetworkError(HTTPClientError):
    """Raised when network connection fails.

    This includes connection errors, DNS resolution failures, and other
    network-level issues.
    """

    def __init__(self, message: str, original_error: Exception | None = None) -> None:
        """Initialize network error.

        Args:
            message: Error message describing the network issue
            original_error: Optional original exception from the HTTP client
        """
        super().__init__(message)
        self.original_error = original_error


class TimeoutError(HTTPClientError):
    """Raised when a request times out.

    This can occur during connection establishment, reading the response,
    or writing the request.
    """

    def __init__(
        self,
        message: str,
        timeout_type: str | None = None,
        original_error: Exception | None = None,
    ) -> None:
        """Initialize timeout error.

        Args:
            message: Error message describing the timeout
            timeout_type: Optional type of timeout (connect, read, write, pool)
            original_error: Optional original exception from the HTTP client
        """
        super().__init__(message)
        self.timeout_type = timeout_type
        self.original_error = original_error



"""Shared operation metadata models."""

from dataclasses import dataclass
from datetime import datetime
from typing import Any, Dict


@dataclass
class OperationMetadata:
    """Shared operation metadata."""

    entity: str
    action: str
    timestamp: datetime
    timing_ms: float | None = None
    status_code: int | None = None
    error_type: str | None = None
    error_message: str | None = None
    params: Dict[str, Any] | None = None



"""
Type definitions for orb connector.
"""
from __future__ import annotations

# Use typing_extensions.TypedDict for Pydantic compatibility
try:
    from typing_extensions import TypedDict, NotRequired
except ImportError:
    from typing import TypedDict, NotRequired  # type: ignore[attr-defined]

from typing import Any, Literal


# ===== NESTED PARAM TYPE DEFINITIONS =====
# Nested parameter schemas discovered during parameter extraction

# ===== OPERATION PARAMS TYPE DEFINITIONS =====

class CustomersListParams(TypedDict):
    """Parameters for customers.list operation"""
    limit: NotRequired[int]
    cursor: NotRequired[str]

class CustomersGetParams(TypedDict):
    """Parameters for customers.get operation"""
    customer_id: str

class SubscriptionsListParams(TypedDict):
    """Parameters for subscriptions.list operation"""
    limit: NotRequired[int]
    cursor: NotRequired[str]
    customer_id: NotRequired[str]
    external_customer_id: NotRequired[str]
    status: NotRequired[str]

class SubscriptionsGetParams(TypedDict):
    """Parameters for subscriptions.get operation"""
    subscription_id: str

class PlansListParams(TypedDict):
    """Parameters for plans.list operation"""
    limit: NotRequired[int]
    cursor: NotRequired[str]

class PlansGetParams(TypedDict):
    """Parameters for plans.get operation"""
    plan_id: str

class InvoicesListParams(TypedDict):
    """Parameters for invoices.list operation"""
    limit: NotRequired[int]
    cursor: NotRequired[str]
    customer_id: NotRequired[str]
    external_customer_id: NotRequired[str]
    subscription_id: NotRequired[str]
    invoice_date_gt: NotRequired[str]
    invoice_date_gte: NotRequired[str]
    invoice_date_lt: NotRequired[str]
    invoice_date_lte: NotRequired[str]
    status: NotRequired[str]

class InvoicesGetParams(TypedDict):
    """Parameters for invoices.get operation"""
    invoice_id: str

# ===== SEARCH TYPES =====

# Sort specification
AirbyteSortOrder = Literal["asc", "desc"]

# ===== CUSTOMERS SEARCH TYPES =====

class CustomersSearchFilter(TypedDict, total=False):
    """Available fields for filtering customers search queries."""
    id: str
    """The unique identifier of the customer"""
    external_customer_id: str | None
    """The ID of the customer in an external system"""
    name: str | None
    """The name of the customer"""
    email: str | None
    """The email address of the customer"""
    created_at: str | None
    """The date and time when the customer was created"""
    payment_provider: str | None
    """The payment provider used by the customer"""
    payment_provider_id: str | None
    """The ID of the customer in the payment provider's system"""
    timezone: str | None
    """The timezone setting of the customer"""
    shipping_address: dict[str, Any] | None
    """The shipping address of the customer"""
    billing_address: dict[str, Any] | None
    """The billing address of the customer"""
    balance: str | None
    """The current balance of the customer"""
    currency: str | None
    """The currency of the customer"""
    auto_collection: bool | None
    """Whether auto collection is enabled"""
    metadata: dict[str, Any] | None
    """Additional metadata for the customer"""


class CustomersInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    id: list[str]
    """The unique identifier of the customer"""
    external_customer_id: list[str]
    """The ID of the customer in an external system"""
    name: list[str]
    """The name of the customer"""
    email: list[str]
    """The email address of the customer"""
    created_at: list[str]
    """The date and time when the customer was created"""
    payment_provider: list[str]
    """The payment provider used by the customer"""
    payment_provider_id: list[str]
    """The ID of the customer in the payment provider's system"""
    timezone: list[str]
    """The timezone setting of the customer"""
    shipping_address: list[dict[str, Any]]
    """The shipping address of the customer"""
    billing_address: list[dict[str, Any]]
    """The billing address of the customer"""
    balance: list[str]
    """The current balance of the customer"""
    currency: list[str]
    """The currency of the customer"""
    auto_collection: list[bool]
    """Whether auto collection is enabled"""
    metadata: list[dict[str, Any]]
    """Additional metadata for the customer"""


class CustomersAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    id: Any
    """The unique identifier of the customer"""
    external_customer_id: Any
    """The ID of the customer in an external system"""
    name: Any
    """The name of the customer"""
    email: Any
    """The email address of the customer"""
    created_at: Any
    """The date and time when the customer was created"""
    payment_provider: Any
    """The payment provider used by the customer"""
    payment_provider_id: Any
    """The ID of the customer in the payment provider's system"""
    timezone: Any
    """The timezone setting of the customer"""
    shipping_address: Any
    """The shipping address of the customer"""
    billing_address: Any
    """The billing address of the customer"""
    balance: Any
    """The current balance of the customer"""
    currency: Any
    """The currency of the customer"""
    auto_collection: Any
    """Whether auto collection is enabled"""
    metadata: Any
    """Additional metadata for the customer"""


class CustomersStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    id: str
    """The unique identifier of the customer"""
    external_customer_id: str
    """The ID of the customer in an external system"""
    name: str
    """The name of the customer"""
    email: str
    """The email address of the customer"""
    created_at: str
    """The date and time when the customer was created"""
    payment_provider: str
    """The payment provider used by the customer"""
    payment_provider_id: str
    """The ID of the customer in the payment provider's system"""
    timezone: str
    """The timezone setting of the customer"""
    shipping_address: str
    """The shipping address of the customer"""
    billing_address: str
    """The billing address of the customer"""
    balance: str
    """The current balance of the customer"""
    currency: str
    """The currency of the customer"""
    auto_collection: str
    """Whether auto collection is enabled"""
    metadata: str
    """Additional metadata for the customer"""


class CustomersSortFilter(TypedDict, total=False):
    """Available fields for sorting customers search results."""
    id: AirbyteSortOrder
    """The unique identifier of the customer"""
    external_customer_id: AirbyteSortOrder
    """The ID of the customer in an external system"""
    name: AirbyteSortOrder
    """The name of the customer"""
    email: AirbyteSortOrder
    """The email address of the customer"""
    created_at: AirbyteSortOrder
    """The date and time when the customer was created"""
    payment_provider: AirbyteSortOrder
    """The payment provider used by the customer"""
    payment_provider_id: AirbyteSortOrder
    """The ID of the customer in the payment provider's system"""
    timezone: AirbyteSortOrder
    """The timezone setting of the customer"""
    shipping_address: AirbyteSortOrder
    """The shipping address of the customer"""
    billing_address: AirbyteSortOrder
    """The billing address of the customer"""
    balance: AirbyteSortOrder
    """The current balance of the customer"""
    currency: AirbyteSortOrder
    """The currency of the customer"""
    auto_collection: AirbyteSortOrder
    """Whether auto collection is enabled"""
    metadata: AirbyteSortOrder
    """Additional metadata for the customer"""


# Entity-specific condition types for customers
class CustomersEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: CustomersSearchFilter


class CustomersNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: CustomersSearchFilter


class CustomersGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: CustomersSearchFilter


class CustomersGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: CustomersSearchFilter


class CustomersLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: CustomersSearchFilter


class CustomersLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: CustomersSearchFilter


class CustomersLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: CustomersStringFilter


class CustomersFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: CustomersStringFilter


class CustomersKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: CustomersStringFilter


class CustomersContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: CustomersAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
CustomersInCondition = TypedDict("CustomersInCondition", {"in": CustomersInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

CustomersNotCondition = TypedDict("CustomersNotCondition", {"not": "CustomersCondition"}, total=False)
"""Negates the nested condition."""

CustomersAndCondition = TypedDict("CustomersAndCondition", {"and": "list[CustomersCondition]"}, total=False)
"""True if all nested conditions are true."""

CustomersOrCondition = TypedDict("CustomersOrCondition", {"or": "list[CustomersCondition]"}, total=False)
"""True if any nested condition is true."""

CustomersAnyCondition = TypedDict("CustomersAnyCondition", {"any": CustomersAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all customers condition types
CustomersCondition = (
    CustomersEqCondition
    | CustomersNeqCondition
    | CustomersGtCondition
    | CustomersGteCondition
    | CustomersLtCondition
    | CustomersLteCondition
    | CustomersInCondition
    | CustomersLikeCondition
    | CustomersFuzzyCondition
    | CustomersKeywordCondition
    | CustomersContainsCondition
    | CustomersNotCondition
    | CustomersAndCondition
    | CustomersOrCondition
    | CustomersAnyCondition
)


class CustomersSearchQuery(TypedDict, total=False):
    """Search query for customers entity."""
    filter: CustomersCondition
    sort: list[CustomersSortFilter]


# ===== SUBSCRIPTIONS SEARCH TYPES =====

class SubscriptionsSearchFilter(TypedDict, total=False):
    """Available fields for filtering subscriptions search queries."""
    id: str
    """The unique identifier of the subscription"""
    created_at: str | None
    """The date and time when the subscription was created"""
    start_date: str | None
    """The date and time when the subscription starts"""
    end_date: str | None
    """The date and time when the subscription ends"""
    status: str | None
    """The current status of the subscription"""
    customer: dict[str, Any] | None
    """The customer associated with the subscription"""
    plan: dict[str, Any] | None
    """The plan associated with the subscription"""
    current_billing_period_start_date: str | None
    """The start date of the current billing period"""
    current_billing_period_end_date: str | None
    """The end date of the current billing period"""
    auto_collection: bool | None
    """Whether auto collection is enabled"""
    net_terms: int | None
    """The net terms for the subscription"""
    metadata: dict[str, Any] | None
    """Additional metadata for the subscription"""


class SubscriptionsInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    id: list[str]
    """The unique identifier of the subscription"""
    created_at: list[str]
    """The date and time when the subscription was created"""
    start_date: list[str]
    """The date and time when the subscription starts"""
    end_date: list[str]
    """The date and time when the subscription ends"""
    status: list[str]
    """The current status of the subscription"""
    customer: list[dict[str, Any]]
    """The customer associated with the subscription"""
    plan: list[dict[str, Any]]
    """The plan associated with the subscription"""
    current_billing_period_start_date: list[str]
    """The start date of the current billing period"""
    current_billing_period_end_date: list[str]
    """The end date of the current billing period"""
    auto_collection: list[bool]
    """Whether auto collection is enabled"""
    net_terms: list[int]
    """The net terms for the subscription"""
    metadata: list[dict[str, Any]]
    """Additional metadata for the subscription"""


class SubscriptionsAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    id: Any
    """The unique identifier of the subscription"""
    created_at: Any
    """The date and time when the subscription was created"""
    start_date: Any
    """The date and time when the subscription starts"""
    end_date: Any
    """The date and time when the subscription ends"""
    status: Any
    """The current status of the subscription"""
    customer: Any
    """The customer associated with the subscription"""
    plan: Any
    """The plan associated with the subscription"""
    current_billing_period_start_date: Any
    """The start date of the current billing period"""
    current_billing_period_end_date: Any
    """The end date of the current billing period"""
    auto_collection: Any
    """Whether auto collection is enabled"""
    net_terms: Any
    """The net terms for the subscription"""
    metadata: Any
    """Additional metadata for the subscription"""


class SubscriptionsStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    id: str
    """The unique identifier of the subscription"""
    created_at: str
    """The date and time when the subscription was created"""
    start_date: str
    """The date and time when the subscription starts"""
    end_date: str
    """The date and time when the subscription ends"""
    status: str
    """The current status of the subscription"""
    customer: str
    """The customer associated with the subscription"""
    plan: str
    """The plan associated with the subscription"""
    current_billing_period_start_date: str
    """The start date of the current billing period"""
    current_billing_period_end_date: str
    """The end date of the current billing period"""
    auto_collection: str
    """Whether auto collection is enabled"""
    net_terms: str
    """The net terms for the subscription"""
    metadata: str
    """Additional metadata for the subscription"""


class SubscriptionsSortFilter(TypedDict, total=False):
    """Available fields for sorting subscriptions search results."""
    id: AirbyteSortOrder
    """The unique identifier of the subscription"""
    created_at: AirbyteSortOrder
    """The date and time when the subscription was created"""
    start_date: AirbyteSortOrder
    """The date and time when the subscription starts"""
    end_date: AirbyteSortOrder
    """The date and time when the subscription ends"""
    status: AirbyteSortOrder
    """The current status of the subscription"""
    customer: AirbyteSortOrder
    """The customer associated with the subscription"""
    plan: AirbyteSortOrder
    """The plan associated with the subscription"""
    current_billing_period_start_date: AirbyteSortOrder
    """The start date of the current billing period"""
    current_billing_period_end_date: AirbyteSortOrder
    """The end date of the current billing period"""
    auto_collection: AirbyteSortOrder
    """Whether auto collection is enabled"""
    net_terms: AirbyteSortOrder
    """The net terms for the subscription"""
    metadata: AirbyteSortOrder
    """Additional metadata for the subscription"""


# Entity-specific condition types for subscriptions
class SubscriptionsEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: SubscriptionsSearchFilter


class SubscriptionsNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: SubscriptionsSearchFilter


class SubscriptionsGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: SubscriptionsSearchFilter


class SubscriptionsGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: SubscriptionsSearchFilter


class SubscriptionsLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: SubscriptionsSearchFilter


class SubscriptionsLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: SubscriptionsSearchFilter


class SubscriptionsLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: SubscriptionsStringFilter


class SubscriptionsFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: SubscriptionsStringFilter


class SubscriptionsKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: SubscriptionsStringFilter


class SubscriptionsContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: SubscriptionsAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
SubscriptionsInCondition = TypedDict("SubscriptionsInCondition", {"in": SubscriptionsInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

SubscriptionsNotCondition = TypedDict("SubscriptionsNotCondition", {"not": "SubscriptionsCondition"}, total=False)
"""Negates the nested condition."""

SubscriptionsAndCondition = TypedDict("SubscriptionsAndCondition", {"and": "list[SubscriptionsCondition]"}, total=False)
"""True if all nested conditions are true."""

SubscriptionsOrCondition = TypedDict("SubscriptionsOrCondition", {"or": "list[SubscriptionsCondition]"}, total=False)
"""True if any nested condition is true."""

SubscriptionsAnyCondition = TypedDict("SubscriptionsAnyCondition", {"any": SubscriptionsAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all subscriptions condition types
SubscriptionsCondition = (
    SubscriptionsEqCondition
    | SubscriptionsNeqCondition
    | SubscriptionsGtCondition
    | SubscriptionsGteCondition
    | SubscriptionsLtCondition
    | SubscriptionsLteCondition
    | SubscriptionsInCondition
    | SubscriptionsLikeCondition
    | SubscriptionsFuzzyCondition
    | SubscriptionsKeywordCondition
    | SubscriptionsContainsCondition
    | SubscriptionsNotCondition
    | SubscriptionsAndCondition
    | SubscriptionsOrCondition
    | SubscriptionsAnyCondition
)


class SubscriptionsSearchQuery(TypedDict, total=False):
    """Search query for subscriptions entity."""
    filter: SubscriptionsCondition
    sort: list[SubscriptionsSortFilter]


# ===== PLANS SEARCH TYPES =====

class PlansSearchFilter(TypedDict, total=False):
    """Available fields for filtering plans search queries."""
    id: str
    """The unique identifier of the plan"""
    created_at: str | None
    """The date and time when the plan was created"""
    name: str | None
    """The name of the plan"""
    description: str | None
    """A description of the plan"""
    status: str | None
    """The status of the plan"""
    currency: str | None
    """The currency of the plan"""
    prices: list[Any] | None
    """The pricing options for the plan"""
    product: dict[str, Any] | None
    """The product associated with the plan"""
    external_plan_id: str | None
    """The external plan ID"""
    metadata: dict[str, Any] | None
    """Additional metadata for the plan"""


class PlansInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    id: list[str]
    """The unique identifier of the plan"""
    created_at: list[str]
    """The date and time when the plan was created"""
    name: list[str]
    """The name of the plan"""
    description: list[str]
    """A description of the plan"""
    status: list[str]
    """The status of the plan"""
    currency: list[str]
    """The currency of the plan"""
    prices: list[list[Any]]
    """The pricing options for the plan"""
    product: list[dict[str, Any]]
    """The product associated with the plan"""
    external_plan_id: list[str]
    """The external plan ID"""
    metadata: list[dict[str, Any]]
    """Additional metadata for the plan"""


class PlansAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    id: Any
    """The unique identifier of the plan"""
    created_at: Any
    """The date and time when the plan was created"""
    name: Any
    """The name of the plan"""
    description: Any
    """A description of the plan"""
    status: Any
    """The status of the plan"""
    currency: Any
    """The currency of the plan"""
    prices: Any
    """The pricing options for the plan"""
    product: Any
    """The product associated with the plan"""
    external_plan_id: Any
    """The external plan ID"""
    metadata: Any
    """Additional metadata for the plan"""


class PlansStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    id: str
    """The unique identifier of the plan"""
    created_at: str
    """The date and time when the plan was created"""
    name: str
    """The name of the plan"""
    description: str
    """A description of the plan"""
    status: str
    """The status of the plan"""
    currency: str
    """The currency of the plan"""
    prices: str
    """The pricing options for the plan"""
    product: str
    """The product associated with the plan"""
    external_plan_id: str
    """The external plan ID"""
    metadata: str
    """Additional metadata for the plan"""


class PlansSortFilter(TypedDict, total=False):
    """Available fields for sorting plans search results."""
    id: AirbyteSortOrder
    """The unique identifier of the plan"""
    created_at: AirbyteSortOrder
    """The date and time when the plan was created"""
    name: AirbyteSortOrder
    """The name of the plan"""
    description: AirbyteSortOrder
    """A description of the plan"""
    status: AirbyteSortOrder
    """The status of the plan"""
    currency: AirbyteSortOrder
    """The currency of the plan"""
    prices: AirbyteSortOrder
    """The pricing options for the plan"""
    product: AirbyteSortOrder
    """The product associated with the plan"""
    external_plan_id: AirbyteSortOrder
    """The external plan ID"""
    metadata: AirbyteSortOrder
    """Additional metadata for the plan"""


# Entity-specific condition types for plans
class PlansEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: PlansSearchFilter


class PlansNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: PlansSearchFilter


class PlansGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: PlansSearchFilter


class PlansGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: PlansSearchFilter


class PlansLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: PlansSearchFilter


class PlansLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: PlansSearchFilter


class PlansLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: PlansStringFilter


class PlansFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: PlansStringFilter


class PlansKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: PlansStringFilter


class PlansContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: PlansAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
PlansInCondition = TypedDict("PlansInCondition", {"in": PlansInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

PlansNotCondition = TypedDict("PlansNotCondition", {"not": "PlansCondition"}, total=False)
"""Negates the nested condition."""

PlansAndCondition = TypedDict("PlansAndCondition", {"and": "list[PlansCondition]"}, total=False)
"""True if all nested conditions are true."""

PlansOrCondition = TypedDict("PlansOrCondition", {"or": "list[PlansCondition]"}, total=False)
"""True if any nested condition is true."""

PlansAnyCondition = TypedDict("PlansAnyCondition", {"any": PlansAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all plans condition types
PlansCondition = (
    PlansEqCondition
    | PlansNeqCondition
    | PlansGtCondition
    | PlansGteCondition
    | PlansLtCondition
    | PlansLteCondition
    | PlansInCondition
    | PlansLikeCondition
    | PlansFuzzyCondition
    | PlansKeywordCondition
    | PlansContainsCondition
    | PlansNotCondition
    | PlansAndCondition
    | PlansOrCondition
    | PlansAnyCondition
)


class PlansSearchQuery(TypedDict, total=False):
    """Search query for plans entity."""
    filter: PlansCondition
    sort: list[PlansSortFilter]


# ===== INVOICES SEARCH TYPES =====

class InvoicesSearchFilter(TypedDict, total=False):
    """Available fields for filtering invoices search queries."""
    id: str
    """The unique identifier of the invoice"""
    created_at: str | None
    """The date and time when the invoice was created"""
    invoice_date: str | None
    """The date of the invoice"""
    due_date: str | None
    """The due date for the invoice"""
    invoice_pdf: str | None
    """The URL to download the PDF version of the invoice"""
    subtotal: str | None
    """The subtotal amount of the invoice"""
    total: str | None
    """The total amount of the invoice"""
    amount_due: str | None
    """The amount due on the invoice"""
    status: str | None
    """The current status of the invoice"""
    memo: str | None
    """Any additional notes or comments on the invoice"""
    paid_at: str | None
    """The date and time when the invoice was paid"""
    issued_at: str | None
    """The date and time when the invoice was issued"""
    hosted_invoice_url: str | None
    """The URL to view the hosted invoice"""
    line_items: list[Any] | None
    """The line items on the invoice"""
    subscription: dict[str, Any] | None
    """The subscription associated with the invoice"""
    customer: dict[str, Any] | None
    """The customer associated with the invoice"""
    currency: str | None
    """The currency of the invoice"""
    invoice_number: str | None
    """The invoice number"""
    metadata: dict[str, Any] | None
    """Additional metadata for the invoice"""


class InvoicesInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    id: list[str]
    """The unique identifier of the invoice"""
    created_at: list[str]
    """The date and time when the invoice was created"""
    invoice_date: list[str]
    """The date of the invoice"""
    due_date: list[str]
    """The due date for the invoice"""
    invoice_pdf: list[str]
    """The URL to download the PDF version of the invoice"""
    subtotal: list[str]
    """The subtotal amount of the invoice"""
    total: list[str]
    """The total amount of the invoice"""
    amount_due: list[str]
    """The amount due on the invoice"""
    status: list[str]
    """The current status of the invoice"""
    memo: list[str]
    """Any additional notes or comments on the invoice"""
    paid_at: list[str]
    """The date and time when the invoice was paid"""
    issued_at: list[str]
    """The date and time when the invoice was issued"""
    hosted_invoice_url: list[str]
    """The URL to view the hosted invoice"""
    line_items: list[list[Any]]
    """The line items on the invoice"""
    subscription: list[dict[str, Any]]
    """The subscription associated with the invoice"""
    customer: list[dict[str, Any]]
    """The customer associated with the invoice"""
    currency: list[str]
    """The currency of the invoice"""
    invoice_number: list[str]
    """The invoice number"""
    metadata: list[dict[str, Any]]
    """Additional metadata for the invoice"""


class InvoicesAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    id: Any
    """The unique identifier of the invoice"""
    created_at: Any
    """The date and time when the invoice was created"""
    invoice_date: Any
    """The date of the invoice"""
    due_date: Any
    """The due date for the invoice"""
    invoice_pdf: Any
    """The URL to download the PDF version of the invoice"""
    subtotal: Any
    """The subtotal amount of the invoice"""
    total: Any
    """The total amount of the invoice"""
    amount_due: Any
    """The amount due on the invoice"""
    status: Any
    """The current status of the invoice"""
    memo: Any
    """Any additional notes or comments on the invoice"""
    paid_at: Any
    """The date and time when the invoice was paid"""
    issued_at: Any
    """The date and time when the invoice was issued"""
    hosted_invoice_url: Any
    """The URL to view the hosted invoice"""
    line_items: Any
    """The line items on the invoice"""
    subscription: Any
    """The subscription associated with the invoice"""
    customer: Any
    """The customer associated with the invoice"""
    currency: Any
    """The currency of the invoice"""
    invoice_number: Any
    """The invoice number"""
    metadata: Any
    """Additional metadata for the invoice"""


class InvoicesStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    id: str
    """The unique identifier of the invoice"""
    created_at: str
    """The date and time when the invoice was created"""
    invoice_date: str
    """The date of the invoice"""
    due_date: str
    """The due date for the invoice"""
    invoice_pdf: str
    """The URL to download the PDF version of the invoice"""
    subtotal: str
    """The subtotal amount of the invoice"""
    total: str
    """The total amount of the invoice"""
    amount_due: str
    """The amount due on the invoice"""
    status: str
    """The current status of the invoice"""
    memo: str
    """Any additional notes or comments on the invoice"""
    paid_at: str
    """The date and time when the invoice was paid"""
    issued_at: str
    """The date and time when the invoice was issued"""
    hosted_invoice_url: str
    """The URL to view the hosted invoice"""
    line_items: str
    """The line items on the invoice"""
    subscription: str
    """The subscription associated with the invoice"""
    customer: str
    """The customer associated with the invoice"""
    currency: str
    """The currency of the invoice"""
    invoice_number: str
    """The invoice number"""
    metadata: str
    """Additional metadata for the invoice"""


class InvoicesSortFilter(TypedDict, total=False):
    """Available fields for sorting invoices search results."""
    id: AirbyteSortOrder
    """The unique identifier of the invoice"""
    created_at: AirbyteSortOrder
    """The date and time when the invoice was created"""
    invoice_date: AirbyteSortOrder
    """The date of the invoice"""
    due_date: AirbyteSortOrder
    """The due date for the invoice"""
    invoice_pdf: AirbyteSortOrder
    """The URL to download the PDF version of the invoice"""
    subtotal: AirbyteSortOrder
    """The subtotal amount of the invoice"""
    total: AirbyteSortOrder
    """The total amount of the invoice"""
    amount_due: AirbyteSortOrder
    """The amount due on the invoice"""
    status: AirbyteSortOrder
    """The current status of the invoice"""
    memo: AirbyteSortOrder
    """Any additional notes or comments on the invoice"""
    paid_at: AirbyteSortOrder
    """The date and time when the invoice was paid"""
    issued_at: AirbyteSortOrder
    """The date and time when the invoice was issued"""
    hosted_invoice_url: AirbyteSortOrder
    """The URL to view the hosted invoice"""
    line_items: AirbyteSortOrder
    """The line items on the invoice"""
    subscription: AirbyteSortOrder
    """The subscription associated with the invoice"""
    customer: AirbyteSortOrder
    """The customer associated with the invoice"""
    currency: AirbyteSortOrder
    """The currency of the invoice"""
    invoice_number: AirbyteSortOrder
    """The invoice number"""
    metadata: AirbyteSortOrder
    """Additional metadata for the invoice"""


# Entity-specific condition types for invoices
class InvoicesEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: InvoicesSearchFilter


class InvoicesNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: InvoicesSearchFilter


class InvoicesGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: InvoicesSearchFilter


class InvoicesGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: InvoicesSearchFilter


class InvoicesLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: InvoicesSearchFilter


class InvoicesLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: InvoicesSearchFilter


class InvoicesLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: InvoicesStringFilter


class InvoicesFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: InvoicesStringFilter


class InvoicesKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: InvoicesStringFilter


class InvoicesContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: InvoicesAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
InvoicesInCondition = TypedDict("InvoicesInCondition", {"in": InvoicesInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

InvoicesNotCondition = TypedDict("InvoicesNotCondition", {"not": "InvoicesCondition"}, total=False)
"""Negates the nested condition."""

InvoicesAndCondition = TypedDict("InvoicesAndCondition", {"and": "list[InvoicesCondition]"}, total=False)
"""True if all nested conditions are true."""

InvoicesOrCondition = TypedDict("InvoicesOrCondition", {"or": "list[InvoicesCondition]"}, total=False)
"""True if any nested condition is true."""

InvoicesAnyCondition = TypedDict("InvoicesAnyCondition", {"any": InvoicesAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all invoices condition types
InvoicesCondition = (
    InvoicesEqCondition
    | InvoicesNeqCondition
    | InvoicesGtCondition
    | InvoicesGteCondition
    | InvoicesLtCondition
    | InvoicesLteCondition
    | InvoicesInCondition
    | InvoicesLikeCondition
    | InvoicesFuzzyCondition
    | InvoicesKeywordCondition
    | InvoicesContainsCondition
    | InvoicesNotCondition
    | InvoicesAndCondition
    | InvoicesOrCondition
    | InvoicesAnyCondition
)


class InvoicesSearchQuery(TypedDict, total=False):
    """Search query for invoices entity."""
    filter: InvoicesCondition
    sort: list[InvoicesSortFilter]



# ===== SEARCH PARAMS =====

class AirbyteSearchParams(TypedDict, total=False):
    """Parameters for Airbyte cache search operations (generic, use entity-specific query types for better type hints)."""
    query: dict[str, Any]
    limit: int
    cursor: str
    fields: list[list[str]]



"""
Blessed Orb connector for Airbyte SDK.

Auto-generated from OpenAPI specification.
"""

from .connector import OrbConnector
from .models import (
    OrbAuthConfig,
    Address,
    CustomerTaxId,
    Customer,
    PaginationMetadata,
    CustomersList,
    SubscriptionCustomer,
    SubscriptionPlan,
    Subscription,
    SubscriptionsList,
    PlanPricesItem,
    PlanProduct,
    Plan,
    PlansList,
    InvoiceLineItemsItem,
    InvoiceSubscription,
    InvoiceCustomer,
    Invoice,
    InvoicesList,
    CustomersListResultMeta,
    SubscriptionsListResultMeta,
    PlansListResultMeta,
    InvoicesListResultMeta,
    OrbCheckResult,
    OrbExecuteResult,
    OrbExecuteResultWithMeta,
    CustomersListResult,
    SubscriptionsListResult,
    PlansListResult,
    InvoicesListResult,
    AirbyteSearchHit,
    AirbyteSearchResult,
    CustomersSearchData,
    CustomersSearchResult,
    SubscriptionsSearchData,
    SubscriptionsSearchResult,
    PlansSearchData,
    PlansSearchResult,
    InvoicesSearchData,
    InvoicesSearchResult
)
from .types import (
    CustomersListParams,
    CustomersGetParams,
    SubscriptionsListParams,
    SubscriptionsGetParams,
    PlansListParams,
    PlansGetParams,
    InvoicesListParams,
    InvoicesGetParams,
    AirbyteSearchParams,
    AirbyteSortOrder,
    CustomersSearchFilter,
    CustomersSearchQuery,
    CustomersCondition,
    SubscriptionsSearchFilter,
    SubscriptionsSearchQuery,
    SubscriptionsCondition,
    PlansSearchFilter,
    PlansSearchQuery,
    PlansCondition,
    InvoicesSearchFilter,
    InvoicesSearchQuery,
    InvoicesCondition
)
from ._vendored.connector_sdk.types import AirbyteHostedAuthConfig

__all__ = [
    "OrbConnector",
    "AirbyteHostedAuthConfig",
    "OrbAuthConfig",
    "Address",
    "CustomerTaxId",
    "Customer",
    "PaginationMetadata",
    "CustomersList",
    "SubscriptionCustomer",
    "SubscriptionPlan",
    "Subscription",
    "SubscriptionsList",
    "PlanPricesItem",
    "PlanProduct",
    "Plan",
    "PlansList",
    "InvoiceLineItemsItem",
    "InvoiceSubscription",
    "InvoiceCustomer",
    "Invoice",
    "InvoicesList",
    "CustomersListResultMeta",
    "SubscriptionsListResultMeta",
    "PlansListResultMeta",
    "InvoicesListResultMeta",
    "OrbCheckResult",
    "OrbExecuteResult",
    "OrbExecuteResultWithMeta",
    "CustomersListResult",
    "SubscriptionsListResult",
    "PlansListResult",
    "InvoicesListResult",
    "AirbyteSearchHit",
    "AirbyteSearchResult",
    "CustomersSearchData",
    "CustomersSearchResult",
    "SubscriptionsSearchData",
    "SubscriptionsSearchResult",
    "PlansSearchData",
    "PlansSearchResult",
    "InvoicesSearchData",
    "InvoicesSearchResult",
    "CustomersListParams",
    "CustomersGetParams",
    "SubscriptionsListParams",
    "SubscriptionsGetParams",
    "PlansListParams",
    "PlansGetParams",
    "InvoicesListParams",
    "InvoicesGetParams",
    "AirbyteSearchParams",
    "AirbyteSortOrder",
    "CustomersSearchFilter",
    "CustomersSearchQuery",
    "CustomersCondition",
    "SubscriptionsSearchFilter",
    "SubscriptionsSearchQuery",
    "SubscriptionsCondition",
    "PlansSearchFilter",
    "PlansSearchQuery",
    "PlansCondition",
    "InvoicesSearchFilter",
    "InvoicesSearchQuery",
    "InvoicesCondition",
]


"""
Pydantic models for orb connector.

This module contains Pydantic models used for authentication configuration
and response envelope types.
"""

from __future__ import annotations

from pydantic import BaseModel, ConfigDict, Field
from typing import TypeVar, Generic, Union, Any

# Authentication configuration

class OrbAuthConfig(BaseModel):
    """API Key Authentication"""

    model_config = ConfigDict(extra="forbid")

    api_key: str
    """Your Orb API key"""

# ===== RESPONSE TYPE DEFINITIONS (PYDANTIC) =====

class Address(BaseModel):
    """Address object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    city: Union[str | None, Any] = Field(default=None)
    country: Union[str | None, Any] = Field(default=None)
    line1: Union[str | None, Any] = Field(default=None)
    line2: Union[str | None, Any] = Field(default=None)
    postal_code: Union[str | None, Any] = Field(default=None)
    state: Union[str | None, Any] = Field(default=None)

class CustomerTaxId(BaseModel):
    """Tax identification information"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    type: Union[str | None, Any] = Field(default=None, description="The type of tax ID")
    """The type of tax ID"""
    value: Union[str | None, Any] = Field(default=None, description="The value of the tax ID")
    """The value of the tax ID"""
    country: Union[str | None, Any] = Field(default=None, description="The country of the tax ID")
    """The country of the tax ID"""

class Customer(BaseModel):
    """Customer object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    external_customer_id: Union[str | None, Any] = Field(default=None)
    name: Union[str | None, Any] = Field(default=None)
    email: Union[str | None, Any] = Field(default=None)
    created_at: Union[str | None, Any] = Field(default=None)
    payment_provider: Union[str | None, Any] = Field(default=None)
    payment_provider_id: Union[str | None, Any] = Field(default=None)
    timezone: Union[str | None, Any] = Field(default=None)
    shipping_address: Union[Any, Any] = Field(default=None)
    billing_address: Union[Any, Any] = Field(default=None)
    balance: Union[str | None, Any] = Field(default=None)
    currency: Union[str | None, Any] = Field(default=None)
    tax_id: Union[CustomerTaxId | None, Any] = Field(default=None)
    auto_collection: Union[bool | None, Any] = Field(default=None)
    metadata: Union[dict[str, Any] | None, Any] = Field(default=None)

class PaginationMetadata(BaseModel):
    """Pagination metadata"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    has_more: Union[bool | None, Any] = Field(default=None)
    next_cursor: Union[str | None, Any] = Field(default=None)

class CustomersList(BaseModel):
    """Paginated list of customers"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[Customer], Any] = Field(default=None)
    pagination_metadata: Union[PaginationMetadata, Any] = Field(default=None)

class SubscriptionCustomer(BaseModel):
    """The customer associated with the subscription"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The customer ID")
    """The customer ID"""
    external_customer_id: Union[str | None, Any] = Field(default=None, description="The external customer ID")
    """The external customer ID"""

class SubscriptionPlan(BaseModel):
    """The plan associated with the subscription"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The plan ID")
    """The plan ID"""
    name: Union[str | None, Any] = Field(default=None, description="The plan name")
    """The plan name"""

class Subscription(BaseModel):
    """Subscription object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    created_at: Union[str | None, Any] = Field(default=None)
    start_date: Union[str | None, Any] = Field(default=None)
    end_date: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    customer: Union[SubscriptionCustomer | None, Any] = Field(default=None)
    plan: Union[SubscriptionPlan | None, Any] = Field(default=None)
    current_billing_period_start_date: Union[str | None, Any] = Field(default=None)
    current_billing_period_end_date: Union[str | None, Any] = Field(default=None)
    active_plan_phase_order: Union[int | None, Any] = Field(default=None)
    fixed_fee_quantity_schedule: Union[list[dict[str, Any]] | None, Any] = Field(default=None)
    price_intervals: Union[list[dict[str, Any]] | None, Any] = Field(default=None)
    redeemed_coupon: Union[dict[str, Any] | None, Any] = Field(default=None)
    default_invoice_memo: Union[str | None, Any] = Field(default=None)
    auto_collection: Union[bool | None, Any] = Field(default=None)
    net_terms: Union[int | None, Any] = Field(default=None)
    invoicing_threshold: Union[str | None, Any] = Field(default=None)
    metadata: Union[dict[str, Any] | None, Any] = Field(default=None)

class SubscriptionsList(BaseModel):
    """Paginated list of subscriptions"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[Subscription], Any] = Field(default=None)
    pagination_metadata: Union[PaginationMetadata, Any] = Field(default=None)

class PlanPricesItem(BaseModel):
    """Nested schema for Plan.prices_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The unique identifier of the price")
    """The unique identifier of the price"""
    name: Union[str | None, Any] = Field(default=None, description="The name of the price")
    """The name of the price"""
    price_type: Union[str | None, Any] = Field(default=None, description="The type of price")
    """The type of price"""
    model_type: Union[str | None, Any] = Field(default=None, description="The model type of the price")
    """The model type of the price"""
    currency: Union[str | None, Any] = Field(default=None, description="The currency of the price")
    """The currency of the price"""

class PlanProduct(BaseModel):
    """The product associated with the plan"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The product ID")
    """The product ID"""
    name: Union[str | None, Any] = Field(default=None, description="The product name")
    """The product name"""

class Plan(BaseModel):
    """Plan object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    created_at: Union[str | None, Any] = Field(default=None)
    name: Union[str | None, Any] = Field(default=None)
    description: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    default_invoice_memo: Union[str | None, Any] = Field(default=None)
    net_terms: Union[int | None, Any] = Field(default=None)
    currency: Union[str | None, Any] = Field(default=None)
    prices: Union[list[PlanPricesItem] | None, Any] = Field(default=None)
    product: Union[PlanProduct | None, Any] = Field(default=None)
    minimum: Union[dict[str, Any] | None, Any] = Field(default=None)
    maximum: Union[dict[str, Any] | None, Any] = Field(default=None)
    discount: Union[dict[str, Any] | None, Any] = Field(default=None)
    trial_config: Union[dict[str, Any] | None, Any] = Field(default=None)
    plan_phases: Union[list[dict[str, Any]] | None, Any] = Field(default=None)
    external_plan_id: Union[str | None, Any] = Field(default=None)
    invoicing_currency: Union[str | None, Any] = Field(default=None)
    metadata: Union[dict[str, Any] | None, Any] = Field(default=None)

class PlansList(BaseModel):
    """Paginated list of plans"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[Plan], Any] = Field(default=None)
    pagination_metadata: Union[PaginationMetadata, Any] = Field(default=None)

class InvoiceLineItemsItem(BaseModel):
    """Nested schema for Invoice.line_items_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The unique identifier of the line item")
    """The unique identifier of the line item"""
    quantity: Union[float | None, Any] = Field(default=None, description="The quantity of the line item")
    """The quantity of the line item"""
    amount: Union[str | None, Any] = Field(default=None, description="The amount of the line item")
    """The amount of the line item"""
    name: Union[str | None, Any] = Field(default=None, description="The name of the line item")
    """The name of the line item"""
    start_date: Union[str | None, Any] = Field(default=None, description="The start date of the line item")
    """The start date of the line item"""
    end_date: Union[str | None, Any] = Field(default=None, description="The end date of the line item")
    """The end date of the line item"""

class InvoiceSubscription(BaseModel):
    """The subscription associated with the invoice"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The subscription ID")
    """The subscription ID"""

class InvoiceCustomer(BaseModel):
    """The customer associated with the invoice"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str | None, Any] = Field(default=None, description="The customer ID")
    """The customer ID"""
    external_customer_id: Union[str | None, Any] = Field(default=None, description="The external customer ID")
    """The external customer ID"""

class Invoice(BaseModel):
    """Invoice object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    created_at: Union[str | None, Any] = Field(default=None)
    invoice_date: Union[str | None, Any] = Field(default=None)
    due_date: Union[str | None, Any] = Field(default=None)
    invoice_pdf: Union[str | None, Any] = Field(default=None)
    subtotal: Union[str | None, Any] = Field(default=None)
    total: Union[str | None, Any] = Field(default=None)
    amount_due: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    memo: Union[str | None, Any] = Field(default=None)
    issue_failed_at: Union[str | None, Any] = Field(default=None)
    sync_failed_at: Union[str | None, Any] = Field(default=None)
    payment_failed_at: Union[str | None, Any] = Field(default=None)
    payment_started_at: Union[str | None, Any] = Field(default=None)
    voided_at: Union[str | None, Any] = Field(default=None)
    paid_at: Union[str | None, Any] = Field(default=None)
    issued_at: Union[str | None, Any] = Field(default=None)
    hosted_invoice_url: Union[str | None, Any] = Field(default=None)
    line_items: Union[list[InvoiceLineItemsItem] | None, Any] = Field(default=None)
    subscription: Union[InvoiceSubscription | None, Any] = Field(default=None)
    customer: Union[InvoiceCustomer | None, Any] = Field(default=None)
    currency: Union[str | None, Any] = Field(default=None)
    discount: Union[dict[str, Any] | None, Any] = Field(default=None)
    minimum: Union[dict[str, Any] | None, Any] = Field(default=None)
    maximum: Union[dict[str, Any] | None, Any] = Field(default=None)
    credit_notes: Union[list[dict[str, Any]] | None, Any] = Field(default=None)
    will_auto_issue: Union[bool | None, Any] = Field(default=None)
    eligible_to_issue_at: Union[str | None, Any] = Field(default=None)
    customer_balance_transactions: Union[list[dict[str, Any]] | None, Any] = Field(default=None)
    auto_collection: Union[dict[str, Any] | None, Any] = Field(default=None)
    invoice_number: Union[str | None, Any] = Field(default=None)
    billing_address: Union[Any, Any] = Field(default=None)
    shipping_address: Union[Any, Any] = Field(default=None)
    metadata: Union[dict[str, Any] | None, Any] = Field(default=None)

class InvoicesList(BaseModel):
    """Paginated list of invoices"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[Invoice], Any] = Field(default=None)
    pagination_metadata: Union[PaginationMetadata, Any] = Field(default=None)

# ===== METADATA TYPE DEFINITIONS (PYDANTIC) =====
# Meta types for operations that extract metadata (e.g., pagination info)

class CustomersListResultMeta(BaseModel):
    """Metadata for customers.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_cursor: Union[str | None, Any] = Field(default=None)

class SubscriptionsListResultMeta(BaseModel):
    """Metadata for subscriptions.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_cursor: Union[str | None, Any] = Field(default=None)

class PlansListResultMeta(BaseModel):
    """Metadata for plans.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_cursor: Union[str | None, Any] = Field(default=None)

class InvoicesListResultMeta(BaseModel):
    """Metadata for invoices.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_cursor: Union[str | None, Any] = Field(default=None)

# ===== CHECK RESULT MODEL =====

class OrbCheckResult(BaseModel):
    """Result of a health check operation.

    Returned by the check() method to indicate connectivity and credential status.
    """
    model_config = ConfigDict(extra="forbid")

    status: str
    """Health check status: 'healthy' or 'unhealthy'."""
    error: str | None = None
    """Error message if status is 'unhealthy', None otherwise."""
    checked_entity: str | None = None
    """Entity name used for the health check."""
    checked_action: str | None = None
    """Action name used for the health check."""


# ===== RESPONSE ENVELOPE MODELS =====

# Type variables for generic envelope models
T = TypeVar('T')
S = TypeVar('S')


class OrbExecuteResult(BaseModel, Generic[T]):
    """Response envelope with data only.

    Used for actions that return data without metadata.
    """
    model_config = ConfigDict(extra="forbid")

    data: T
    """Response data containing the result of the action."""


class OrbExecuteResultWithMeta(OrbExecuteResult[T], Generic[T, S]):
    """Response envelope with data and metadata.

    Used for actions that return both data and metadata (e.g., pagination info).
    """
    meta: S
    """Metadata about the response (e.g., pagination cursors, record counts)."""

# ===== SEARCH DATA MODELS =====
# Entity-specific Pydantic models for search result data

# Type variable for search data generic
D = TypeVar('D')

class CustomersSearchData(BaseModel):
    """Search result data for customers entity."""
    model_config = ConfigDict(extra="allow")

    id: str = None
    """The unique identifier of the customer"""
    external_customer_id: str | None = None
    """The ID of the customer in an external system"""
    name: str | None = None
    """The name of the customer"""
    email: str | None = None
    """The email address of the customer"""
    created_at: str | None = None
    """The date and time when the customer was created"""
    payment_provider: str | None = None
    """The payment provider used by the customer"""
    payment_provider_id: str | None = None
    """The ID of the customer in the payment provider's system"""
    timezone: str | None = None
    """The timezone setting of the customer"""
    shipping_address: dict[str, Any] | None = None
    """The shipping address of the customer"""
    billing_address: dict[str, Any] | None = None
    """The billing address of the customer"""
    balance: str | None = None
    """The current balance of the customer"""
    currency: str | None = None
    """The currency of the customer"""
    auto_collection: bool | None = None
    """Whether auto collection is enabled"""
    metadata: dict[str, Any] | None = None
    """Additional metadata for the customer"""


class SubscriptionsSearchData(BaseModel):
    """Search result data for subscriptions entity."""
    model_config = ConfigDict(extra="allow")

    id: str = None
    """The unique identifier of the subscription"""
    created_at: str | None = None
    """The date and time when the subscription was created"""
    start_date: str | None = None
    """The date and time when the subscription starts"""
    end_date: str | None = None
    """The date and time when the subscription ends"""
    status: str | None = None
    """The current status of the subscription"""
    customer: dict[str, Any] | None = None
    """The customer associated with the subscription"""
    plan: dict[str, Any] | None = None
    """The plan associated with the subscription"""
    current_billing_period_start_date: str | None = None
    """The start date of the current billing period"""
    current_billing_period_end_date: str | None = None
    """The end date of the current billing period"""
    auto_collection: bool | None = None
    """Whether auto collection is enabled"""
    net_terms: int | None = None
    """The net terms for the subscription"""
    metadata: dict[str, Any] | None = None
    """Additional metadata for the subscription"""


class PlansSearchData(BaseModel):
    """Search result data for plans entity."""
    model_config = ConfigDict(extra="allow")

    id: str = None
    """The unique identifier of the plan"""
    created_at: str | None = None
    """The date and time when the plan was created"""
    name: str | None = None
    """The name of the plan"""
    description: str | None = None
    """A description of the plan"""
    status: str | None = None
    """The status of the plan"""
    currency: str | None = None
    """The currency of the plan"""
    prices: list[Any] | None = None
    """The pricing options for the plan"""
    product: dict[str, Any] | None = None
    """The product associated with the plan"""
    external_plan_id: str | None = None
    """The external plan ID"""
    metadata: dict[str, Any] | None = None
    """Additional metadata for the plan"""


class InvoicesSearchData(BaseModel):
    """Search result data for invoices entity."""
    model_config = ConfigDict(extra="allow")

    id: str = None
    """The unique identifier of the invoice"""
    created_at: str | None = None
    """The date and time when the invoice was created"""
    invoice_date: str | None = None
    """The date of the invoice"""
    due_date: str | None = None
    """The due date for the invoice"""
    invoice_pdf: str | None = None
    """The URL to download the PDF version of the invoice"""
    subtotal: str | None = None
    """The subtotal amount of the invoice"""
    total: str | None = None
    """The total amount of the invoice"""
    amount_due: str | None = None
    """The amount due on the invoice"""
    status: str | None = None
    """The current status of the invoice"""
    memo: str | None = None
    """Any additional notes or comments on the invoice"""
    paid_at: str | None = None
    """The date and time when the invoice was paid"""
    issued_at: str | None = None
    """The date and time when the invoice was issued"""
    hosted_invoice_url: str | None = None
    """The URL to view the hosted invoice"""
    line_items: list[Any] | None = None
    """The line items on the invoice"""
    subscription: dict[str, Any] | None = None
    """The subscription associated with the invoice"""
    customer: dict[str, Any] | None = None
    """The customer associated with the invoice"""
    currency: str | None = None
    """The currency of the invoice"""
    invoice_number: str | None = None
    """The invoice number"""
    metadata: dict[str, Any] | None = None
    """Additional metadata for the invoice"""


# ===== GENERIC SEARCH RESULT TYPES =====

class AirbyteSearchHit(BaseModel, Generic[D]):
    """A single search result with typed data."""
    model_config = ConfigDict(extra="allow")

    id: str | None = None
    """Unique identifier for the record."""
    score: float | None = None
    """Relevance score for the match."""
    data: D
    """The matched record data."""


class AirbyteSearchResult(BaseModel, Generic[D]):
    """Result from Airbyte cache search operations with typed hits."""
    model_config = ConfigDict(extra="allow")

    hits: list[AirbyteSearchHit[D]] = Field(default_factory=list)
    """List of matching records."""
    next_cursor: str | None = None
    """Cursor for fetching the next page of results."""
    took_ms: int | None = None
    """Time taken to execute the search in milliseconds."""


# ===== ENTITY-SPECIFIC SEARCH RESULT TYPE ALIASES =====

CustomersSearchResult = AirbyteSearchResult[CustomersSearchData]
"""Search result type for customers entity."""

SubscriptionsSearchResult = AirbyteSearchResult[SubscriptionsSearchData]
"""Search result type for subscriptions entity."""

PlansSearchResult = AirbyteSearchResult[PlansSearchData]
"""Search result type for plans entity."""

InvoicesSearchResult = AirbyteSearchResult[InvoicesSearchData]
"""Search result type for invoices entity."""



# ===== OPERATION RESULT TYPE ALIASES =====

# Concrete type aliases for each operation result.
# These provide simpler, more readable type annotations than using the generic forms.

CustomersListResult = OrbExecuteResultWithMeta[list[Customer], CustomersListResultMeta]
"""Result type for customers.list operation with data and metadata."""

SubscriptionsListResult = OrbExecuteResultWithMeta[list[Subscription], SubscriptionsListResultMeta]
"""Result type for subscriptions.list operation with data and metadata."""

PlansListResult = OrbExecuteResultWithMeta[list[Plan], PlansListResultMeta]
"""Result type for plans.list operation with data and metadata."""

InvoicesListResult = OrbExecuteResultWithMeta[list[Invoice], InvoicesListResultMeta]
"""Result type for invoices.list operation with data and metadata."""

